@@ -27,6 +27,7 @@ Commands:
 
 Options:
   --help -h          Display this message
+  --version          Show version and exit
 
 Type 'brownie <command> --help' for specific options and more information about
 each command."""
@@ -36,10 +37,17 @@ def main():
 
     print(f"Brownie v{__version__} - Python development framework for Ethereum\n")
 
+    if "--version" in sys.argv:
+        sys.exit()
+
     if len(sys.argv) < 2 or sys.argv[1].startswith("-"):
         # this call triggers a SystemExit
         docopt(__doc__, ["brownie", "-h"])
 
+    if "-i" in sys.argv:
+        # a small kindness to ipython users
+        sys.argv[sys.argv.index("-i")] = "-I"
+
     cmd = sys.argv[1]
     cmd_list = [i.stem for i in Path(__file__).parent.glob("[!_]*.py")]
     if cmd not in cmd_list:
