@@ -29,6 +29,7 @@ class SourceNoteBook(ttk.Notebook):
             if path.suffix in (".sol", ".vy"):
                 self.add(path)
         self.set_visible([])
+        self.bind("<<NotebookTabChanged>>", self.on_tab_change)
 
     def add(self, path):
         path = Path(path)
@@ -62,6 +63,11 @@ class SourceNoteBook(ttk.Notebook):
         frame._visible = True
         super().add(frame, text=f"   {label}   ")
 
+    def on_tab_change(self, event):
+        if self.select():
+            tab = event.widget.tab("current")["text"]
+            self.root.toolbar.report.set_values(Path(tab).stem.strip())
+
     def set_visible(self, labels):
         labels = [Path(i).name for i in labels]
         for label in [i._label for i in self._frames]:
