@@ -1,6 +1,7 @@
 #!/usr/bin/python3
 
 import json
+import sys
 import threading
 import time
 from collections.abc import Iterator
@@ -11,8 +12,15 @@ from typing import Any, Dict, List, Optional, Tuple, Union
 import eth_account
 import eth_keys
 import rlp
+from eip712.messages import EIP712Message, _hash_eip191_message
+from eth_account._utils.signing import sign_message_hash
+from eth_account.datastructures import SignedMessage
+from eth_account.messages import defunct_hash_message
 from eth_utils import keccak
+from eth_utils.applicators import apply_formatters_to_dict
 from hexbytes import HexBytes
+from web3 import HTTPProvider, IPCProvider
+from web3.exceptions import InvalidTransaction
 
 from brownie._config import CONFIG, _get_data_folder
 from brownie._singleton import _Singleton
@@ -172,7 +180,7 @@ class Accounts(metaclass=_Singleton):
             return new_accounts[0]
         return new_accounts
 
-    def load(self, filename: str = None) -> Union[List, "LocalAccount"]:
+    def load(self, filename: str = None, password: str = None) -> Union[List, "LocalAccount"]:
         """
         Load a local account from a keystore file.
 
@@ -180,6 +188,9 @@ class Accounts(metaclass=_Singleton):
         ---------
         filename: str
             Keystore filename. If `None`, returns a list of available keystores.
+        password: str
+            Password to unlock the keystore. If `None`, password is entered via
+            a getpass prompt.
 
         Returns
         -------
@@ -192,7 +203,7 @@ class Accounts(metaclass=_Singleton):
         filename = str(filename)
         json_file = Path(filename).expanduser()
 
-        if not json_file.exists():
+        if not json_file.exists() or json_file.is_dir():
             temp_json_file = json_file.with_suffix(".json")
             if temp_json_file.exists():
                 json_file = temp_json_file
@@ -206,7 +217,8 @@ class Accounts(metaclass=_Singleton):
 
         with json_file.open() as fp:
             priv_key = web3.eth.account.decrypt(
-                json.load(fp), getpass("Enter the password to unlock this account: ")
+                json.load(fp),
+                password or getpass(f'Enter password for "{json_file.stem}": '),
             )
         return self.add(priv_key)
 
@@ -263,6 +275,54 @@ class Accounts(metaclass=_Singleton):
         """
         self._accounts.clear()
 
+    def connect_to_clef(self, uri: str = None, timeout: int = 120) -> None:
+        """
+        Connect to Clef and import open accounts.
+
+        Clef is an account signing utility packaged with Geth, which can be
+        used to interact with HW wallets in Brownie. Before calling this
+        function, Clef must be running in another command prompt.
+
+        Arguments
+        ---------
+        uri : str
+            IPC path or http url to use to connect to clef. If None is given,
+            uses the default IPC path on Unix systems or localhost on Windows.
+        timeout : int
+            The number of seconds to wait on a clef request before raising a
+            timeout exception.
+        """
+        provider = None
+        if uri is None:
+            if sys.platform == "win32":
+                uri = "http://localhost:8550/"
+            else:
+                uri = Path.home().joinpath(".clef/clef.ipc").as_posix()
+        try:
+            if Path(uri).exists():
+                provider = IPCProvider(uri, timeout=timeout)
+        except OSError:
+            if uri is not None and uri.startswith("http"):
+                provider = HTTPProvider(uri, {"timeout": timeout})
+        if provider is None:
+            raise ValueError("Unknown URI, must be IPC socket path or URL starting with 'http'")
+
+        response = provider.make_request("account_list", [])
+        if "error" in response:
+            raise ValueError(response["error"]["message"])
+
+        for address in response["result"]:
+            if to_address(address) not in self._accounts:
+                self._accounts.append(ClefAccount(address, provider))
+
+    def disconnect_from_clef(self) -> None:
+        """
+        Disconnect from Clef.
+
+        Removes all `ClefAccount` objects from the container.
+        """
+        self._accounts = [i for i in self._accounts if not isinstance(i, ClefAccount)]
+
 
 class PublicKeyAccount:
     """Class for interacting with an Ethereum account where you do not control
@@ -334,7 +394,7 @@ class _PrivateKeyAccount(PublicKeyAccount):
         super().__init__(addr)
 
     def _pending_nonce(self) -> int:
-        tx_from_sender = history.from_sender(self.address)
+        tx_from_sender = sorted(history.from_sender(self.address), key=lambda k: k.nonce)
         if len(tx_from_sender) == 0:
             return self.nonce
 
@@ -354,7 +414,7 @@ class _PrivateKeyAccount(PublicKeyAccount):
         self,
         to: Optional["Account"],
         amount: int,
-        gas_price: int,
+        gas_price: Optional[int],
         gas_buffer: Optional[float],
         data: Optional[str] = None,
     ) -> int:
@@ -364,7 +424,7 @@ class _PrivateKeyAccount(PublicKeyAccount):
 
         if isinstance(gas_limit, bool) or gas_limit in (None, "auto"):
             gas_buffer = gas_buffer or CONFIG.active_network["settings"]["gas_buffer"]
-            gas_limit = self.estimate_gas(to, amount, gas_price, data or "")
+            gas_limit = self.estimate_gas(to, amount, 0, data or "")
             if gas_limit > 21000 and gas_buffer != 1:
                 gas_limit = Wei(gas_limit * gas_buffer)
                 return min(gas_limit, Chain().block_gas_limit)
@@ -397,7 +457,10 @@ class _PrivateKeyAccount(PublicKeyAccount):
 
     def _check_for_revert(self, tx: Dict) -> None:
         try:
-            web3.eth.call(dict((k, v) for k, v in tx.items() if v))
+            # remove gas price related values to avoid issues post-EIP1559
+            # https://github.com/ethereum/go-ethereum/pull/23027
+            skip_keys = {"gasPrice", "maxFeePerGas", "maxPriorityFeePerGas"}
+            web3.eth.call({k: v for k, v in tx.items() if k not in skip_keys and v})
         except ValueError as exc:
             msg = exc.args[0]["message"] if isinstance(exc.args[0], dict) else str(exc)
             raise ValueError(
@@ -413,6 +476,8 @@ class _PrivateKeyAccount(PublicKeyAccount):
         gas_limit: Optional[int] = None,
         gas_buffer: Optional[float] = None,
         gas_price: Optional[int] = None,
+        max_fee: Optional[int] = None,
+        priority_fee: Optional[int] = None,
         nonce: Optional[int] = None,
         required_confs: int = 1,
         allow_revert: bool = None,
@@ -430,55 +495,31 @@ class _PrivateKeyAccount(PublicKeyAccount):
             amount: Amount of ether to send with transaction, in wei.
             gas_limit: Gas limit of the transaction.
             gas_buffer: Multiplier to apply to gas limit.
-            gas_price: Gas price of the transaction.
+            gas_price: Gas price of legacy transaction.
+            max_fee: Max fee per gas of dynamic fee transaction.
+            priority_fee: Max priority fee per gas of dynamic fee transaction.
             nonce: Nonce to use for the transaction.
 
         Returns:
             * Contract instance if the transaction confirms and the contract exists
             * TransactionReceipt if the transaction is pending or reverts
         """
-        if gas_limit and gas_buffer:
-            raise ValueError("Cannot set gas_limit and gas_buffer together")
-
         data = contract.deploy.encode_input(*args)
-        if silent is None:
-            silent = bool(CONFIG.mode == "test" or CONFIG.argv["silent"])
-        with self._lock:
-            try:
-                gas_price, gas_strategy, gas_iter = self._gas_price(gas_price)
-                gas_limit = Wei(gas_limit) or self._gas_limit(
-                    None, amount, gas_price, gas_buffer, data
-                )
-                txid = self._transact(  # type: ignore
-                    {
-                        "from": self.address,
-                        "value": Wei(amount),
-                        "nonce": nonce if nonce is not None else self._pending_nonce(),
-                        "gasPrice": gas_price,
-                        "gas": gas_limit,
-                        "data": HexBytes(data),
-                    },
-                    allow_revert,
-                )
-                exc, revert_data = None, None
-            except ValueError as e:
-                exc = VirtualMachineError(e)
-                if not hasattr(exc, "txid"):
-                    raise exc from None
-                txid = exc.txid
-                revert_data = (exc.revert_msg, exc.pc, exc.revert_type)
-
-            receipt = TransactionReceipt(
-                txid,
-                self,
-                silent=silent,
-                required_confs=required_confs,
-                is_blocking=False,
-                name=contract._name + ".constructor",
-                revert_data=revert_data,
-            )
-
-        receipt = self._await_confirmation(receipt, required_confs, gas_strategy, gas_iter)
+        receipt, exc = self._make_transaction(
+            None,
+            amount,
+            gas_limit,
+            gas_buffer,
+            gas_price,
+            max_fee,
+            priority_fee,
+            data,
+            nonce,
+            contract._name + ".constructor",
+            required_confs,
+            allow_revert,
+            silent,
+        )
 
         add_thread = threading.Thread(target=contract._add_from_tx, args=(receipt,), daemon=True)
         add_thread.start()
@@ -495,6 +536,8 @@ class _PrivateKeyAccount(PublicKeyAccount):
                         "gas_limit": gas_limit,
                         "gas_buffer": gas_buffer,
                         "gas_price": gas_price,
+                        "max_fee": max_fee,
+                        "priority_fee": priority_fee,
                     },
                 ),
                 daemon=True,
@@ -545,7 +588,7 @@ class _PrivateKeyAccount(PublicKeyAccount):
             "data": HexBytes(data or ""),
         }
         if gas_price is not None:
-            tx["gasPrice"] = gas_price
+            tx["gasPrice"] = web3.toHex(gas_price)
         try:
             return web3.eth.estimate_gas(tx)
         except ValueError as exc:
@@ -569,6 +612,8 @@ class _PrivateKeyAccount(PublicKeyAccount):
         gas_limit: Optional[int] = None,
         gas_buffer: Optional[float] = None,
         gas_price: Optional[int] = None,
+        max_fee: Optional[int] = None,
+        priority_fee: Optional[int] = None,
         data: str = None,
         nonce: Optional[int] = None,
         required_confs: int = 1,
@@ -583,7 +628,9 @@ class _PrivateKeyAccount(PublicKeyAccount):
             amount: Amount of ether to send, in wei.
             gas_limit: Gas limit of the transaction.
             gas_buffer: Multiplier to apply to gas limit.
-            gas_price: Gas price of the transaction.
+            gas_price: Gas price of legacy transaction.
+            max_fee: Max fee per gas of dynamic fee transaction.
+            priority_fee: Max priority fee per gas of dynamic fee transaction.
             nonce: Nonce to use for the transaction.
             data: Hexstring of data to include in transaction.
             silent: Toggles console verbosity.
@@ -591,23 +638,104 @@ class _PrivateKeyAccount(PublicKeyAccount):
         Returns:
             TransactionReceipt object
         """
+
+        receipt, exc = self._make_transaction(
+            to,
+            amount,
+            gas_limit,
+            gas_buffer,
+            gas_price,
+            max_fee,
+            priority_fee,
+            data or "",
+            nonce,
+            "",
+            required_confs,
+            allow_revert,
+            silent,
+        )
+
+        if rpc.is_active():
+            undo_thread = threading.Thread(
+                target=Chain()._add_to_undo_buffer,
+                args=(
+                    receipt,
+                    self.transfer,
+                    [],
+                    {
+                        "to": to,
+                        "amount": amount,
+                        "gas_limit": gas_limit,
+                        "gas_buffer": gas_buffer,
+                        "gas_price": gas_price,
+                        "max_fee": max_fee,
+                        "priority_fee": priority_fee,
+                        "data": data,
+                    },
+                ),
+                daemon=True,
+            )
+            undo_thread.start()
+
+        receipt._raise_if_reverted(exc)
+        return receipt
+
+    def _make_transaction(
+        self,
+        to: Optional["Account"],
+        amount: int,
+        gas_limit: Optional[int],
+        gas_buffer: Optional[float],
+        gas_price: Optional[int],
+        max_fee: Optional[int],
+        priority_fee: Optional[int],
+        data: str,
+        nonce: Optional[int],
+        fn_name: str,
+        required_confs: int,
+        allow_revert: Optional[bool],
+        silent: Optional[bool],
+    ) -> Tuple[TransactionReceipt, Optional[Exception]]:
+        # shared logic for `transfer` and `deploy`
         if gas_limit and gas_buffer:
             raise ValueError("Cannot set gas_limit and gas_buffer together")
         if silent is None:
             silent = bool(CONFIG.mode == "test" or CONFIG.argv["silent"])
+
+        if gas_price is None:
+            # if gas price is not explicitly set, load the default max fee and priority fee
+            if max_fee is None:
+                max_fee = CONFIG.active_network["settings"]["max_fee"] or None
+            if priority_fee is None:
+                priority_fee = CONFIG.active_network["settings"]["priority_fee"] or None
+
+        if priority_fee == "auto":
+            priority_fee = Chain().priority_fee
+
+        try:
+            # if max fee and priority fee are not set, use gas price
+            if max_fee is None and priority_fee is None:
+                gas_price, gas_strategy, gas_iter = self._gas_price(gas_price)
+            else:
+                gas_strategy, gas_iter = None, None
+            gas_limit = Wei(gas_limit) or self._gas_limit(
+                to, amount, gas_price or max_fee, gas_buffer, data
+            )
+        except ValueError as e:
+            raise VirtualMachineError(e) from None
+
         with self._lock:
-            gas_price, gas_strategy, gas_iter = self._gas_price(gas_price)
-            gas_limit = Wei(gas_limit) or self._gas_limit(to, amount, gas_price, gas_buffer, data)
+            # we use a lock here to prevent nonce issues when sending many tx's at once
             tx = {
                 "from": self.address,
                 "value": Wei(amount),
                 "nonce": nonce if nonce is not None else self._pending_nonce(),
-                "gasPrice": gas_price,
-                "gas": gas_limit,
-                "data": HexBytes(data or ""),
+                "gas": web3.toHex(gas_limit),
+                "data": HexBytes(data),
             }
             if to:
                 tx["to"] = to_address(str(to))
+            tx = _apply_fee_to_tx(tx, gas_price, max_fee, priority_fee)
             try:
                 txid = self._transact(tx, allow_revert)  # type: ignore
                 exc, revert_data = None, None
@@ -618,32 +746,17 @@ class _PrivateKeyAccount(PublicKeyAccount):
                 txid = exc.txid
                 revert_data = (exc.revert_msg, exc.pc, exc.revert_type)
 
-            receipt = TransactionReceipt(
-                txid,
-                self,
-                required_confs=required_confs,
-                is_blocking=False,
-                silent=silent,
-                revert_data=revert_data,
-            )
-
+        receipt = TransactionReceipt(
+            txid,
+            self,
+            silent=silent,
+            required_confs=required_confs,
+            is_blocking=False,
+            name=fn_name,
+            revert_data=revert_data,
+        )
         receipt = self._await_confirmation(receipt, required_confs, gas_strategy, gas_iter)
-
-        if rpc.is_active():
-            undo_thread = threading.Thread(
-                target=Chain()._add_to_undo_buffer,
-                args=(
-                    receipt,
-                    self.transfer,
-                    (to, amount, gas_limit, gas_buffer, gas_price, data, None),
-                    {},
-                ),
-                daemon=True,
-            )
-            undo_thread.start()
-
-        receipt._raise_if_reverted(exc)
-        return receipt
+        return receipt, exc
 
     def _await_confirmation(
         self,
@@ -757,6 +870,50 @@ class LocalAccount(_PrivateKeyAccount):
             json.dump(encrypted, fp)
         return str(json_file)
 
+    def sign_defunct_message(self, message: str) -> SignedMessage:
+        """Signs an `EIP-191` using this account's private key.
+
+        Args:
+            message: An text
+
+        Returns:
+            An eth_account `SignedMessage` instance.
+        """
+        msg_hash_bytes = defunct_hash_message(text=message)
+        eth_private_key = eth_keys.keys.PrivateKey(HexBytes(self.private_key))
+        (v, r, s, eth_signature_bytes) = sign_message_hash(eth_private_key, msg_hash_bytes)
+        return SignedMessage(
+            messageHash=msg_hash_bytes,
+            r=r,
+            s=s,
+            v=v,
+            signature=HexBytes(eth_signature_bytes),
+        )
+
+    def sign_message(self, message: EIP712Message) -> SignedMessage:
+        """Signs an `EIP712Message` using this account's private key.
+
+        Args:
+            message: An `EIP712Message` instance.
+
+        Returns:
+            An eth_account `SignedMessage` instance.
+        """
+        # some of this code is from:
+        # https://github.com/ethereum/eth-account/blob/00e7b10/eth_account/account.py#L577
+        # https://github.com/ethereum/eth-account/blob/00e7b10/eth_account/account.py#L502
+        msg_hash_bytes = HexBytes(_hash_eip191_message(message.signable_message))
+        assert len(msg_hash_bytes) == 32, "The message hash must be exactly 32-bytes"
+        eth_private_key = eth_keys.keys.PrivateKey(HexBytes(self.private_key))
+        (v, r, s, eth_signature_bytes) = sign_message_hash(eth_private_key, msg_hash_bytes)
+        return SignedMessage(
+            messageHash=msg_hash_bytes,
+            r=r,
+            s=s,
+            v=v,
+            signature=HexBytes(eth_signature_bytes),
+        )
+
     def _transact(self, tx: Dict, allow_revert: bool) -> None:
         if allow_revert is None:
             allow_revert = bool(CONFIG.network_type == "development")
@@ -765,3 +922,74 @@ class LocalAccount(_PrivateKeyAccount):
         tx["chainId"] = web3.chain_id
         signed_tx = self._acct.sign_transaction(tx).rawTransaction  # type: ignore
         return web3.eth.send_raw_transaction(signed_tx)
+
+
+class ClefAccount(_PrivateKeyAccount):
+
+    """
+    Class for interacting with an Ethereum account where signing is handled in Clef.
+    """
+
+    def __init__(self, address: str, provider: Union[HTTPProvider, IPCProvider]) -> None:
+        self._provider = provider
+        super().__init__(address)
+
+    def _transact(self, tx: Dict, allow_revert: bool) -> None:
+        if allow_revert is None:
+            allow_revert = bool(CONFIG.network_type == "development")
+        if not allow_revert:
+            self._check_for_revert(tx)
+
+        formatters = {
+            "nonce": web3.toHex,
+            "gasPrice": web3.toHex,
+            "gas": web3.toHex,
+            "value": web3.toHex,
+            "chainId": web3.toHex,
+            "data": web3.toHex,
+            "from": to_address,
+        }
+        if "to" in tx:
+            formatters["to"] = to_address
+
+        tx["chainId"] = web3.chain_id
+        tx = apply_formatters_to_dict(formatters, tx)
+
+        response = self._provider.make_request("account_signTransaction", [tx])
+        if "error" in response:
+            raise ValueError(response["error"]["message"])
+        return web3.eth.send_raw_transaction(response["result"]["raw"])
+
+
+def _apply_fee_to_tx(
+    tx: Dict,
+    gas_price: Optional[int] = None,
+    max_fee: Optional[int] = None,
+    priority_fee: Optional[int] = None,
+) -> Dict:
+    tx = tx.copy()
+
+    if gas_price is not None:
+        if max_fee or priority_fee:
+            raise ValueError("gas_price and (max_fee, priority_fee) are mutually exclusive")
+        tx["gasPrice"] = web3.toHex(gas_price)
+        return tx
+
+    if priority_fee is None:
+        raise InvalidTransaction("priority_fee must be defined")
+    priority_fee = Wei(priority_fee)
+
+    # no max_fee specified, infer from base_fee
+    if max_fee is None:
+        base_fee = Chain().base_fee
+        max_fee = base_fee * 2 + priority_fee
+    else:
+        max_fee = Wei(max_fee)
+
+    if priority_fee > max_fee:
+        raise InvalidTransaction("priority_fee must not exceed max_fee")
+
+    tx["maxFeePerGas"] = web3.toHex(max_fee)
+    tx["maxPriorityFeePerGas"] = web3.toHex(priority_fee)
+    tx["type"] = web3.toHex(2)
+    return tx
