@@ -33,6 +33,7 @@ class Web3(_Web3):
         self._chain_uri: Optional[str] = None
         self._custom_middleware: Set = set()
         self._supports_traces = None
+        self._chain_id: Optional[int] = None
 
     def _remove_middlewares(self) -> None:
         for middleware in self._custom_middleware:
@@ -105,6 +106,7 @@ class Web3(_Web3):
             self._genesis_hash = None
             self._chain_uri = None
             self._supports_traces = None
+            self._chain_id = None
             self._remove_middlewares()
 
     def isConnected(self) -> bool:
@@ -147,7 +149,7 @@ class Web3(_Web3):
         if self.provider is None:
             raise ConnectionError("web3 is not currently connected")
         if self._genesis_hash is None:
-            self._genesis_hash = self.eth.getBlock(0)["hash"].hex()[2:]
+            self._genesis_hash = self.eth.get_block(0)["hash"].hex()[2:]
         return self._genesis_hash
 
     @property
@@ -155,12 +157,22 @@ class Web3(_Web3):
         if self.provider is None:
             raise ConnectionError("web3 is not currently connected")
         if self.genesis_hash not in _chain_uri_cache:
-            block_number = max(self.eth.blockNumber - 16, 0)
-            block_hash = self.eth.getBlock(block_number)["hash"].hex()[2:]
+            block_number = max(self.eth.block_number - 16, 0)
+            block_hash = self.eth.get_block(block_number)["hash"].hex()[2:]
             chain_uri = f"blockchain://{self.genesis_hash}/block/{block_hash}"
             _chain_uri_cache[self.genesis_hash] = chain_uri
         return _chain_uri_cache[self.genesis_hash]
 
+    @property
+    def chain_id(self) -> int:
+        # chain ID is needed each time we a sign a transaction, however we
+        # cache it after the first request to avoid redundant RPC calls
+        if self.provider is None:
+            raise ConnectionError("web3 is not currently connected")
+        if self._chain_id is None:
+            self._chain_id = self.eth.chain_id
+        return self._chain_id
+
 
 def _expand_environment_vars(uri: str) -> str:
     if "$" not in uri:
@@ -195,7 +207,7 @@ def _resolve_address(domain: str) -> str:
 
 
 web3 = Web3()
-web3.eth.setGasPriceStrategy(rpc_gas_price_strategy)
+web3.eth.set_gas_price_strategy(rpc_gas_price_strategy)
 
 try:
     with _get_path().open() as fp:
